/*
    userver -- (pronounced you-server or micro-server).
    This file is part of the userver, a high-performance web server designed for
    performance experiments.
          
    This file is Copyright (C) 2005  David Pariag

    Authors: David Pariag <db2paria@cs.uwaterloo.ca>
    See AUTHORS file for list of contributors to the project.
  
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/


#include "distributions.h"
#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/time.h>

#define WEIBULL_A    (1.46)
#define WEIBULL_B    (0.382)
#define PARETO_ALPHA (1.5)

double getParetoValue(double alpha) {
  double random;
  
  assert( alpha != 0.0);
  random = ((double) rand()) / ((double) RAND_MAX);
  return pow((1.0 - random), (-1.0 / alpha));  
}


double getWeibullValue(double a, double b) {
  double random;
  double retval;
  assert(b != 0.0);
  random = ((double) rand()) / ((double) RAND_MAX);
  return  b * (pow( (-1.0 * log(1.0 - random)), (1.0 / a)));

}

//#define TESTING
#ifdef TESTING
int main() {
  double value;
  double total;
  int iterations;
  int i;
  double max;
  struct timeval tv;
  
  srand((unsigned int) time(NULL));
  
  iterations = 100000000;
  total = 0.0;
  value = 0.0;
  max = 0.0;
  printf("\nGenerating %d numbers according to the Pareto distribution\n\n", iterations);
  for( i = 0; i < iterations; i++ ) {
    value = getParetoValue(PARETO_ALPHA);
    total += value;
    if( value > max ) max = value;
    //    printf("%2.4f ", value);
    //    if( (i+1) % 10 == 0 ) printf("\n");
  }
  printf("Mean = %2.4f\n", total / ((double) iterations));
  printf("Max = %8.4f\n\n", max );
  
  total = 0.0;
  value = 0.0;
  max = 0.0;
  printf("Generating %d numbers according to the Weibull distribution\n\n", iterations);
  for( i = 0; i < iterations; i++ ) {
    value = getWeibullValue( WEIBULL_A , WEIBULL_B );
    if ( value > 1.0 ) value = 1.0;  
    total += value;
    if( value > max ) max = value;
    //    printf("%2.4f ", value);
    //    if( (i+1) % 10 == 0 ) printf("\n");
  }
  printf("Mean = %2.4f\n", total / ((double) iterations));
  printf("Max = %8.4f\n\n", max );
  return 0;
}
#endif /* TESTING */
