/*
    userver -- (pronounced you-server or micro-server).
    This file is part of the userver, a high-performance web server designed for
    performance experiments.
          
    This file is Copyright (C) 2005  David Pariag

    Authors: David Pariag <db2paria@cs.uwaterloo.ca>
    See AUTHORS file for list of contributors to the project.
  
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/



/*******************************************************************************
 * All static SPECweb99 uris are of the form:
 *    /some-path/dir<xxxxx>/class<y>_<z>
 * where x,y, and z are digits. For simplicity, we store the three 
 * numbers and use them to generate the uri when it is needed.
 *
 * Dynamic uris fall into one of the following categories:
 * 
 * DYNAMIC GET
 *   /fcgi-bin/specweb99-fcgi.pl?/static-uri
 *   where static-uri is defined above
 *
 * DYNAMIC POST
 *  /specweb99-fcgi.pl? method=POST contents="urlroot=/file_set/&dir=00000&class=0&num=1&client=10018\r\n"
 * 
 * 
 *******************************************************************************/

#ifndef SESSION_H
#define SESSION_H

/* Represent the different types of request that are present in
   SPECweb99 workloads */
typedef enum req_type { STATIC = 0, DYNAMIC_GET, DYNAMIC_GET_AD, DYNAMIC_POST, NUM_REQ_TYPES} req_type;

/* SPECweb99 cookies are of the form: my_cookie=user_id=10001&last_ad=268 */
typedef struct cookie {
  int userid;
  int lastad;
} cookie;


/* Represents one URI (request) in a session */
typedef struct uri { 
  int dir;            /* The directory number   */
  int class;          /* The class number (0-3) */
  int file;           /* The file number (0-8)  */
  float think;        /* The think time for this uri */
  cookie * mycookie;  /* Cookie used in dynamic GETs with ad rotation*/
  req_type type;      /* What kind of request is this? */
} uri;

/* A session is essentially a list of URIs */
typedef struct session {
  int length;      /* The number of URIs in this session */
  int bytes;       /* The number of bytes requested in the session */
  uri * uris;      /* An array of uris */
  struct session * next;  /* Use this pointer to construct session lists */
} session;


typedef struct session_list {
  session * head; /* Pointer to first element in the list */
  int length;     /* Number of sessions in the list */
} session_list;

/********************************************************************************/
/*                               FUNCTIONS                                      */
/********************************************************************************/


/* Allocate memory for a new session */
session * new_session(int length);

/* Free the memory associated with this session */
void free_session(session * tofree);

/* Add the given session to the specified session list */
void add_session(session_list * s, session * toadd);

/* Remove the first session from the specified list  */
session * remove_first_session(session_list * s);

#endif /* SESSION_H */
