/*
    WatPipe.
    This file is part of the WatPipe, a high-performance web server designed for
    performance experiments.
          
    This file is Copyright (C) 2010  Ashif Harji

    Authors: Ashif Harji <asharji@cs.uwaterloo.ca>
    See AUTHORS file for list of contributors to the project.
  
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/


#include <sys/param.h> 
#include <pthread.h>
#include <stdio.h>
#include <signal.h>

#include "common.h"
#include "interest_set.h"
#include "options.h"

#include "seda.h"
#include "stats.h"
#include "util.h"
#include "sock_listener.h"

#include <sys/epoll.h>

#include "state.h"
#include "syscalls.h"

#include <sys/time.h>
#include <sys/resource.h>

#include <errno.h>

//#undef NDEBUG
//#define LOCAL_DEBUG
#include "local_debug.h"
#include "debug.h"

extern fd_set *writable, *rdfds1, *wrfds1, *rdfds;
extern fd_set *closableA;
extern struct epoll_event *epoll_write_results;
    
extern int interest_set_is_set(int sd, enum which_set_enum which_set);
extern int epoll_is_readable(uint32_t evt);
extern int epoll_is_writable(uint32_t evt);
extern int epoll_is_hungup(uint32_t evt);

#define mb()	asm volatile("mfence" ::: "memory")

readSelQueue *ReadSelQueue;
acceptBlock **AcceptBlock;
selectRead *SelectRead;
selectWrite *SelectWrite;
writeQueue *WriteQueue;
frQueue **FRQueue;
frQueue **FRQueueReader;
reader **Readers;

int writer::num_fds = 0;
extern volatile int readerSleep;
extern int readerBroadcast;

//int numcpus = 4;

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

inline void mergeFD( fd_set *source, fd_set *dest) {
//    int tmasks = howmany( max_fds + 1, NFDBITS );		// total number of masks in fd set
    int tmasks = howmany( max_sd + 1, NFDBITS );		// total number of masks in fd set

    for ( int i = 0; i < tmasks; i += 1 ) {
	dest->fds_bits[i] |= source->fds_bits[i];
	source->fds_bits[i] = 0;
    } // for
} //mergeFD


void do_affinity(int aff_mask) {
    cpu_set_t tmp_mask;
    CPU_ZERO(&tmp_mask);
    tmp_mask.__bits[0] = aff_mask;
    int rc = setaffinity(pthread_self(), sizeof(tmp_mask), &tmp_mask);

    if (rc < 0) {
	printf("set_userver_cpu_affinity: unable to set processor affinity mask\n");
        printf("set_userver_cpu_affinity: call returns %d errno = %d\n", rc, errno);
        perror("unable to sched_setaffinity");
        exit(1);
    }
    rc = getaffinity(pthread_self(), sizeof(tmp_mask), &tmp_mask);
    if (rc < 0) {
	printf("set_userver_cpu_affinity: unable to get processor affinity mask\n");
        printf("set_userver_cpu_affinity: call returns %d errno = %d\n", rc, errno);
        perror("unable to sched_getaffinity");
        exit(1);
    }
    print_cpuset_values("Effective CPU mask: \n", &tmp_mask);
    printf("\n");

}

#define DOMASK 0

void acceptBlock::main() {
#if 0
    do_affinity(0x1 << (id%numcpus) );
#endif

    do_plock dp(&l);

    ldbg("acceptBlock task: %d\n", (int)tid);

    while( true ) {
	ldbg("accept loop\n");
	if( numIdle == 0 ) num_accept_idle += 1;

	num_accept_loops += 1;
	socket_new_conn( listen_sd, id );

	// I suspect there is a race condition between deciding to go to sleep and having 
	// all the signals arrive before you are actually asleep
	// It is unlikely, but a timeout on the wait would probably help
	while ( numIdle == 0 ) {
	    num_accept_probes += 1;
	    blocked = true;
#if 0
	    struct timeval now;
	    struct timespec timeout;
	    gettimeofday(&now, NULL);
	    timeout.tv_sec = now.tv_sec;
	    timeout.tv_nsec = now.tv_usec * 1000 + 1;	
	    pthread_cond_timedwait( &wq, &l, &timeout );
#else
	    pthread_cond_wait( &wq, &l );
#endif
   	    blocked = false;
	} // if

    } // while

} // acceptBlock::main



void selectRead::main() {
#if DOMASK
    //do_affinity(0x2);
#endif

    do_plock dp(&l);
    
    // start read poll and perform reads
    if ( options.use_epoll ) {
	epoll_loop(true, false);
    } else {
	select_loop(true, false );
    } // if
} // selectRead::main

void selectWrite::main() {

    do_plock dp(&l);
    
    // start write poll and perform reads
    if ( options.use_epoll ) {
	epoll_loop( false, true);
    } else {
	select_loop( false, true );   
    } // if

} // selectWrite::main


void writer::main() {
#if 1
    do_affinity(0x1 << (id%numcpus));
#endif

    do_plock dp(&l);

    int num_written = 0;
    
    num_written = 0;
    int sd = -1;
    int closed = 0;
    while ( true ) {
	num_writer_loops++;	
	if( options.epoll_edge_triggered ) {
	    sd = FRQueue[id]->forwardPickup(sd);
	} else {
	    sd = WriteQueue->writePickup(sd);
	} // if
	assert(sd != 0);
	closed = do_writable(sd);

	if ((num_closed_intermW == 0) && (get_fsm_state(sd) != FSM_WRITING_REPLY) ) {
	    // add to readSelQueue
	    FRQueueReader[id]->forwardDrop(sd);
	    sd = -1;
	} // if

	// don't pass back sd if connection was closed
	if( num_closed_intermW > 0 ) {
	    num_closed_intermW = 0;
	    sd = -1;
	} // if

	num_written += 1;
    } // while

} // writer::main


void reader::main() {
#if 1
    do_affinity(0x1 << (id%numcpus) );
#endif

    do_plock dp(&l);
    sleep(1);

    int num_written = 0;
    
    num_written = 0;
    int sd = -1;
    int closed = 0;
    while ( true ) {
	if( options.epoll_edge_triggered ) {
	    sd = FRQueueReader[id]->forwardPickup(sd);
	} else {
	    sd = ReadSelQueue->readPickup(sd);
	} // if
	assert(sd != 0);

	struct info *ip = info_ptr_no_assert(sd);
	if( ip->sd == -1) {
	    socket_setup(info_listener[sock_listener_min].sd, sd);
	} // if

	assert( ip->sd > 0 );
	closed = do_readable(sd);


	if ((num_closed_intermW == 0) && (get_fsm_state(sd) == FSM_WRITING_REPLY) ) {
	    // add to writeQueue
	    FRQueue[id]->newWritesDrop(sd);
	    sd = -1;
	} // if

	// don't pass back sd if connection was closed
	if( num_closed_intermW > 0 ) {
	    num_closed_intermW = 0;
	    sd = -1;

	} // if

    } // while

} // reader::main



// When running edge-triggered epoll, readable sds are given directly to the
// reader task but not necessarily read immediately
void writeQueue::eventsDrop(int num) {
    extern struct epoll_event *epoll_write_results;
//    do_plock dp(&l);
    int sd;
    struct info * ip = NULL;
    int af_queue = -1;
    bool wait = false;
    int signal[numcpus];

    memset( signal, 0, numcpus*sizeof(int) );

    // add writable fds to queue
    for (int i=0; i < num; i++) {
      sd = epoll_write_results[i].data.fd;
      uint32_t evt = epoll_write_results[i].events;

      // may not have ownership of sd if edge triggered epoll
      if( !options.epoll_edge_triggered && epoll_is_hungup( evt ) ) {
	  /* epoll reported an error, so close the connection down */
	  ip = info_ptr(sd);
	  do_close(ip, true);
	  if (evt & EPOLLERR) num_epoll_epollerr++;
	  if (evt & EPOLLHUP) num_epoll_epollhup++;

      } else if ( epoll_is_writable( evt )) {
	    af_queue = ReadSelQueue->Admin_info[sd].affinity_queue;
	    if ( FRQueue[af_queue]->eventDrop(sd, wait) ) {
		signal[af_queue] += 1;
	    } // if
	}	
    } // for

    for ( int i=0; i < numcpus; i++ ) {
	if ( signal[i] ) {
//	    pthread_cond_broadcast( &FRQueue[i]->TaskWait);
	    FRQueue[i]->signalTask();
	} // if
    } // for

    usleep(5000);
} // writeQueue::eventsDrop



int frQueue::forwardPickup(int sd) {
    do_plock dp(&l);
    int ret = -1;

    if( sd >= 0 ) {
	if ( sd_state[sd] & EPOLLIN ) {
	    sd_state[sd] &= ~(EPOLLIN);
	    ret = sd;
	    if( taskWaiting > 0 && numAvail > 0) pthread_cond_signal(&TaskWaitOther);		
	    return ret;
	} else {
	    FD_SET( sd, &blockedTasks);
	} //endif
    } // if
	
    int ev_sd = -1;

    while( true ) {
	// wait for new sd

	for( ; frontEvents != backEvents;  ) {
	    ev_sd = queueEvents[backEvents];
	    backEvents = (backEvents + 1)%queueSize;
	    if ( FD_ISSET(ev_sd, &blockedTasks) ) {
		FD_CLR(ev_sd, &blockedTasks);
		queueAccept[frontAccept] = ev_sd;
		frontAccept = (frontAccept + 1)%queueSize;
		sd_state[ev_sd] |= EPOLLIN;		
		numAvail += 1;
	    } else {
		sd_state[ev_sd] |= EPOLLIN;		
	    } // if
	} // for

	while( frontAccept != backAccept ) {
	    int old = backAccept;
	    ret = queueAccept[backAccept];
	    backAccept = (backAccept + 1) % queueSize;
	    assert( (old + 1)%queueSize == backAccept );
	    numAvail -= 1;
	    if ( sd_state[ret] & EPOLLIN ) {
		sd_state[ret] &= ~(EPOLLIN);
		if( taskWaiting > 0 && numAvail > 0 ) pthread_cond_signal(&TaskWaitOther);
		return ret;
	    } else {
		FD_SET( ret, &blockedTasks);
	    } // if
	} // while

	while( frontForward != backForward ) {
	    int old = backForward;
	    ret = queueForward[backForward];
	    backForward = (backForward + 1) % queueSize;
	    assert( (old + 1)%queueSize == backForward );
	    numAvail -= 1;
	    if ( sd_state[ret] & EPOLLIN ) {
		sd_state[ret] &= ~(EPOLLIN);
		if( taskWaiting > 0 && numAvail > 0) pthread_cond_signal(&TaskWaitOther);
		return ret;
	    } else {
		FD_SET( ret, &blockedTasks);
	    } // if
	} // while


	taskWaiting += 1;
	if ( taskWaiting > maxWaiting ) maxWaiting = taskWaiting;

	if ( reader ) {
	    do_temp_punlock dl(&l);
	    FRQueue[id]->signalTask();
	} // if
	if ( !reader && taskWaitingOther == maxWaiting - 1) {
	    do_temp_punlock dl( &l);
	    FRQueueReader[id]->signalTask();
	} // if

	if ( taskWaitingOther == maxWaiting - 1 ) {
	    pthread_cond_broadcast(&dropWait);
	    broadcast = 0;
	    onTaskWaitingQ = true;
	    pthread_cond_wait( &TaskWait, &l );
	    onTaskWaitingQ = false;
	    broadcast = 0;
	} else {
	    taskWaitingOther += 1;
	    pthread_cond_wait(&TaskWaitOther, &l);
	    taskWaitingOther -= 1;
	} // if
	taskWaiting -= 1;

    } // while
    return ret;

} // frQueue::forwardPickup

void frQueue::forwardDrop(int sd) {
    {
	do_plock dp(&l);
	int old = frontForward;
	queueForward[frontForward] = sd;
	frontForward = (frontForward + 1) % queueSize;
	assert( (old + 1)%queueSize == frontForward );
	numAvail += 1;
	//pthread_cond_signal( &TaskWait );
    }
} // frQueue::forwardDrop

int frQueue::returnPickup(bool &writable) {
//    do_plock dp(&l);
    mb();
    int ret = -1;
    int old = backReturn;
    if ( frontReturn == backReturn ) return ret;
    ret = queueReturn[backReturn];
    backReturn = (backReturn + 1) % queueSize;
    assert( (old + 1)%queueSize == backReturn);
    if ( ret < 0 ) {
	writable = false;
	ret = -ret;
    } else {
	writable = true;
    } // if

    return ret;
} // frQueue::returnPickup


void frQueue::acceptDrop(int sd) {
    {
	do_plock dp(&l);
	int old = frontAccept;
	queueAccept[frontAccept] = sd;
	frontAccept = (frontAccept + 1) % queueSize;
	assert( (old + 1)%queueSize == frontAccept);
	numAvail += 1;

	if( onTaskWaitingQ ) {
	    if ( broadcast == 0 ) {
		broadcast = 1;
		pthread_cond_signal(&TaskWait);
	    } // if
	} else {
	    pthread_cond_wait(&dropWait, &l);
	} // if
    } // plock

} // frQueue::acceptDrop

int frQueue::acceptPickup() {
//    do_plock dp(&l);
    mb();
    int ret = -1;
    int old = backAccept;
    if ( frontAccept == backAccept ) return ret;
    ret = queueAccept[backAccept];
     backAccept = (backAccept + 1) % queueSize;
    assert( (old + 1)%queueSize == backAccept);

    mb();
    return ret;
} // frQueue::acceptPickup

int frQueue::eventDrop(int sd, bool wait) {
    {
//	do_plock dp(&l);
	int old = frontEvents;
	queueEvents[frontEvents] = sd;
	frontEvents = (frontEvents + 1) % queueSize;
	assert( (old + 1)%queueSize == frontEvents);
	return true;
    } // plock
} // frQueue::eventDrop

void frQueue::signalTask() {
    do_plock dp(&l);
    if ( broadcast == 0 ) {
	broadcast = 1;
	if( onTaskWaitingQ ) {
	    pthread_cond_broadcast( &TaskWait );
	} else {
	    pthread_cond_signal( &TaskWaitOther);
	} // if
    } //if 
} // frQueue::signalTask

void frQueue::newWritesDrop(int sd) {
    {
	do_plock dp(&l);
	int old = frontAccept;
	queueAccept[frontAccept] = sd;
	frontAccept = (frontAccept + 1) % queueSize;
	assert( (old + 1)%queueSize == frontAccept);
	numAvail += 1;
	sd_state[sd] = EPOLLIN;

	if( onTaskWaitingQ ) {
	    readerBroadcast += 1;
	    if ( broadcast == 0 ) {
		broadcast = 1;
		pthread_cond_signal(&TaskWait);
	    } // if
	} else {
	    readerSleep += 1;
#if 0
            struct timeval now;
            struct timespec timeout;
            gettimeofday(&now, NULL);
            timeout.tv_sec = now.tv_sec;
            timeout.tv_nsec = now.tv_usec * 1000 + 25000;
            pthread_cond_timedwait( &dropWait, &l, &timeout );
#else
	    if ( !options.blocking_sendfile && numTasks <= numcpus ) pthread_cond_wait(&dropWait, &l);
#endif
            if ( taskWaitingOther ) pthread_cond_signal(&TaskWaitOther);
	} // if
    } // plock

} // frQueue::newWritesDrop



// When running edge-triggered epoll, readable sds are given directly to the
// reader task but not necessarily read immediately
void readSelQueue::eventsDrop(int num) {
    extern struct epoll_event *epoll_read_results;
//    do_plock dp(&l);
    int sd;
    struct info * ip = NULL;
    int af_queue = -1;
    int wait = false;
    int signal[numcpus];
    
    memset( signal, 0, numcpus*sizeof(int) );

    // add readable fds to queue
    for ( int i=0; i<num; i++ ) {
      sd = epoll_read_results[i].data.fd;
      uint32_t evt = epoll_read_results[i].events;

      // may not have ownership of sd if edge triggered epoll
      if( !options.epoll_edge_triggered && epoll_is_hungup( evt ) ) {
	  /* epoll reported an error, so close the connection down */
	  ip = info_ptr(sd);
	  do_close(ip, true);
	  if (evt & EPOLLERR) num_epoll_epollerr++;
	  if (evt & EPOLLHUP) num_epoll_epollhup++;

      } else if ( epoll_is_readable( evt )) {
	  af_queue = Admin_info[sd].affinity_queue;
	  if( FRQueueReader[af_queue]->eventDrop(sd, wait) ) {
	      signal[af_queue] += 1;	      
	  } // if

	}	
    } // for

    for ( int i=0; i < numcpus; i++ ) {
	if ( signal[i] ) {
	    FRQueueReader[i]->signalTask();
	} // if
    } // for
    
    usleep(1900);
} // readSelQueue::eventsDrop



void readSelQueue::setAffinityQueue(int sd, int queue) {
    int affinity_queue;
    int subnet_id;
    struct admin_info *aip = &Admin_info[sd];

    affinity_queue = queue%numcpus;
    subnet_id = queue;

    aip->affinity_queue = affinity_queue;
    aip->subnet_id = subnet_id;
} // readSelQueue::setAffinityQueue

