/*
    WatPipe.
    This file is part of the WatPipe, a high-performance web server designed for
    performance experiments.
          
    This file is Copyright (C) 2010  Ashif Harji

    Authors: Ashif Harji <asharji@cs.uwaterloo.ca>
    See AUTHORS file for list of contributors to the project.
  
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/

#include <sys/epoll.h>

extern fd_set *closableS;

class acceptBlock :public ptask {
    pthread_cond_t  dropWait;
    void doPickup();
    int id;
    int listen_sd;
    
  public:
    bool waiting;
    bool blocked;
    pthread_cond_t wq;
    int numIdle;
    int curConns;
    
    void main();
    acceptBlock( int id, int l_sd ) : id(id), listen_sd( l_sd ) {
	pthread_cond_init( &wq, NULL);
	waiting = false;
	blocked = false;

	// each acceptor gets an equal number of connectins
	numIdle = maxconns /  options.ip_addr_count;
	curConns = 0;
	
	pthread_create(&tid, NULL, start, this);	
	printf("created acceptBlock task %d, this: %p, id: %d, listen_sd: %d\n", (int)tid, this, id, listen_sd);

    } // acceptBlock

    ~acceptBlock() {
    	pthread_join( tid, &ret);
    } // ~acceptBlock
    
    void dropoff(fd_set **f );
    int getRead(fd_set *f );
    
}; // acceptBlock


class selectRead : public ptask {
    bool waiting;
    
  public:
    pthread_cond_t wq;
    
    void main();
    void getWrite(fd_set *fw, fd_set *fr );
    void mergeWrite();
    selectRead() {
	pthread_cond_init( &wq, NULL);
	waiting = false;
	pthread_create(&tid, NULL, start, this);
	printf("created task %d, this: %p\n", (int)tid, this);
    } // selectRead
    
    ~selectRead() {
    	pthread_join( tid, &ret);
    }
    
}; // selectRead

class selectWrite : public ptask {
    
  public:
    pthread_cond_t wq;
    bool waiting;
    
    void main();

    selectWrite(){
	waiting = false;
	pthread_cond_init( &wq, NULL);
	int rc = pthread_create(&tid, NULL, start, this);	
	if( rc != 0 ) perror("pthread_create");
	printf("created task %d, this: %p\n", (int)tid, this);
    }
    
    ~selectWrite() {
    	pthread_join( tid, &ret);
    }
    
}; // selectWrite

class writer : public ptask {
    int id;
    selectWrite &SelectWrite;
    
  public:
    static int num_fds;
    
    void main();
    writer( int id, selectWrite &sw ) : id(id%numcpus), SelectWrite(sw) {
	pthread_attr_t   stackSize;
	size_t      size = 1024*1024*2;
	
	pthread_attr_init (&stackSize);
	pthread_attr_setstacksize (&stackSize, size);

	int rc = pthread_create(&tid, &stackSize, start, this);
	if( rc != 0 ) perror("pthread_create");
	//printf("%d: created writer %d, this: %p\n", rc, id, this);
    }
    
    ~writer() {
    	pthread_join( tid, &ret);
    }
    
}; // writer


class reader : public ptask {
    int id;
    selectRead &SelectRead;
    fd_set blockedTasks;
    
  public:
#if 0
    infot info;
#endif
    
    void main();
    reader( int id, selectRead &sr ) : id(id%numcpus), SelectRead(sr) {
	int size = sizeof(fd_set);
	memset(&blockedTasks, 0, size);
	int rc = pthread_create(&tid, NULL, start, this);	
	if( rc != 0 ) perror("pthread_create");
	//printf("created task %d, this: %p\n", (int)tid, this);
    }
    
    ~reader() {
    	pthread_join( tid, &ret);
    }
    
}; // reader


class writeQueue {
    pthread_cond_t  pollTaskWait;
    pthread_mutex_t l;
    pthread_mutex_t l2;
    int *queueEvents, *queueWritable;
    int num_closed_internal;
    fd_set blockedTasks;
    int writerWaitCount;
    
  public:
    pthread_cond_t  writerWait;
    int *sd_write_state;
    int frontEvents, frontWritable;
    int backEvents, backWritable;
    
    writeQueue(){
	pthread_mutex_init( &l, NULL );
	pthread_mutex_init( &l2, NULL );
	pthread_cond_init( &writerWait, NULL );
	pthread_cond_init( &pollTaskWait, NULL );

	// events from epoll wait
	queueEvents = new int[maxconns];
	queueWritable = new int[maxconns];
	frontEvents = backEvents = 0;
	frontWritable = backWritable = 0;
	num_closed_internal = 0;
	writerWaitCount = 0;
	int size = sizeof(fd_set);
	memset(&blockedTasks, 0, size);

	sd_write_state = new int[max_fds];
    }

    void readDrop(int num);
    void eventsDrop(int num);
    void readDrop_et(int sd);
    int writePickup(int old_sd);
    int writePickup_et(int old_sd, bool writeBlocked);
    void writeAdmin_et();
};


class frQueue {
    pthread_mutex_t l;
    pthread_mutex_t l2;
    pthread_mutex_t l3;
    int *queueForward, *queueReturn, *queueAccept, *queueEvents;
    int queueSize;
    int id;
    fd_set blockedTasks;
    int *sd_state;
    int broadcast;
    bool reader;
    int numAvail;
    
  public:
    
    pthread_cond_t  TaskWait;
    pthread_cond_t  TaskWaitOther;
    pthread_cond_t *returnWait;
    pthread_cond_t  dropWait;
    bool locked;
    int taskWaiting, maxWaiting;
    int taskWaitingOther;
    int frontForward, frontReturn, frontClose, frontAccept, frontEvents;
    int backForward, backReturn, backAccept, backEvents;
    bool onTaskWaitingQ;
    
  frQueue(int id, bool rd ) : id(id), reader(rd) {
	pthread_mutex_init( &l, NULL );
	pthread_mutex_init( &l2, NULL );
	pthread_mutex_init( &l3, NULL );
	pthread_cond_init( &TaskWait, NULL );
	pthread_cond_init( &TaskWaitOther, NULL );
	pthread_cond_init( &dropWait, NULL );
	returnWait = NULL;
	queueSize = maxconns + 1;
	queueForward = new int[queueSize];
	queueReturn = new int[queueSize];
	queueAccept = new int[queueSize];
	queueEvents = new int[queueSize];
	sd_state = new int[max_fds];

	frontForward = backForward = 0;
	frontReturn = backReturn = 0;
	frontAccept = backAccept = 0;
	frontEvents = backEvents = 0;
        locked = false;
	
	int size = sizeof(fd_set);
	memset(&blockedTasks, 0, size);

	taskWaiting = taskWaitingOther = 0;
	maxWaiting = 1;
	broadcast = 0;
	numAvail = 0;
	onTaskWaitingQ = false;
    }

    int forwardPickup(int sd);
    void forwardDrop(int sd);
    int returnPickup(bool &writable);
    void acceptDrop(int sd);
    int acceptPickup();
    int eventDrop(int sd, bool wait);
    void newWritesDrop(int sd);
    void signalTask();
};


class readSelQueue {
    pthread_mutex_t l, l2;
    int *queueEvents;
    int *queueReadable;
    int *queueReadableAgain;
    int num_closed_internal;
    fd_set blockedTasks;
    
  public:
    pthread_cond_t  readerWait, readerWait1;
    admin_info *Admin_info;
    int frontEvents, frontReadable, frontReadableAgain;
    int backEvents, backReadable, backReadableAgain;
    bool waiting;
    
    readSelQueue(){
	pthread_mutex_init( &l, NULL );
	pthread_mutex_init( &l2, NULL );
	pthread_cond_init( &readerWait, NULL );
	pthread_cond_init( &readerWait1, NULL );

	// events from epoll_wait
	queueEvents = new int[maxconns];
	// if edge triggered, readable sds
	queueReadable = new int[maxconns];
	queueReadableAgain = new int[maxconns];
	frontEvents = backEvents = 0;
	frontReadable = backReadable = 0;
	frontReadableAgain = backReadableAgain = 0;
	num_closed_internal = 0;

	int size = sizeof(fd_set);
	memset(&blockedTasks, 0, size);
	
	Admin_info = new admin_info[max_fds];
	waiting = false;
    }

    void readDrop(int num);
    void readDrop_et(int sd);
    void readDropAgain_et(int sd);
    void eventsDrop(int num);
    int readPickup(int num);
    int readPickup_et(int sd, bool readBlocked);
    void readAdmin_et();
    
    void setAffinityQueue(int sd, int queue);
    int length() {
	return (frontEvents - backEvents + maxconns)%maxconns;
    } // length
    
}; // class readSelQueue


//extern acceptBlock *AcceptBlock;
extern selectRead *SelectRead;
extern selectWrite *SelectWrite;
extern writeQueue *WriteQueue;
extern readSelQueue *ReadSelQueue;
extern frQueue **FRQueue;
extern frQueue **FRQueueReader;
extern reader **Readers;
extern acceptBlock **AcceptBlock;

