/*
    WatPipe.
    This file is part of the WatPipe, a high-performance web server designed for
    performance experiments.
          
    This file is Copyright (C) 2010  Ashif Harji

    Authors: Ashif Harji <asharji@cs.uwaterloo.ca>
    See AUTHORS file for list of contributors to the project.
  
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of the
    License, or (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
  
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA
*/

#include <sys/param.h> 
#include <pthread.h>
#include <stdio.h>
#include <signal.h>

#include "common.h"
#include "interest_set.h"
#include "options.h"

#include "seda.h"
#include "stats.h"
#include "util.h"
#include "sock_listener.h"

#include <sys/epoll.h>

#include "state.h"
#include "syscalls.h"

#include <sys/time.h>
#include <sys/resource.h>

#include <errno.h>

//#undef NDEBUG
//#define LOCAL_DEBUG
#include "local_debug.h"
#include "debug.h"

extern fd_set *writable, *rdfds1, *wrfds1, *rdfds;
extern fd_set *closableA;
extern struct epoll_event *epoll_write_results;
    
extern int interest_set_is_set(int sd, enum which_set_enum which_set);
extern int epoll_is_readable(uint32_t evt);
extern int epoll_is_writable(uint32_t evt);
extern int epoll_is_hungup(uint32_t evt);

#define mb()	asm volatile("mfence" ::: "memory")

extern volatile int readerSleep;
extern int readerBroadcast;

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

inline void mergeFD( fd_set *source, fd_set *dest) {
//    int tmasks = howmany( max_fds + 1, NFDBITS );		// total number of masks in fd set
    int tmasks = howmany( max_sd + 1, NFDBITS );		// total number of masks in fd set

    for ( int i = 0; i < tmasks; i += 1 ) {
	dest->fds_bits[i] |= source->fds_bits[i];
	source->fds_bits[i] = 0;
    } // for
} //mergeFD

static int connections;

void acceptBlock::dropoff(fd_set **f ){
    do_plock dp(&l);

    struct info *ip;
    fd_set *maskptr;

    maskptr = (fd_set *)*f;
    for (int sd = 0; sd <= max_sd; sd += 1) {
	if ( FD_ISSET(sd, maskptr) ) {
	    //if ( event_set_is_readable(sd) || FD_ISSET(sd, rdfds)) uAbort("accept close assertion fail");
	    ip = info_ptr(sd);
	    if( ip != NULL) {
		do_close(ip, true);
		FD_CLR(sd, maskptr);
	    } // if
	}// if
    } // for
} // acceptBlock::dropoff

void acceptBlock::doPickup( ){
} // acceptBlock::doPickup
	    
int acceptBlock::getRead(fd_set *f ){
    waiting = true;
    do_plock dp(&l);

    ldbg("do getRead: num_interm: %d\n", num_closed_interm);
    waiting = false;

    if ( blocked ) pthread_cond_signal( &wq );

    //int adjustment = total_connections - num_closed_interm;
    int adjustment = connections;
    numIdle += num_closed_interm;

    num_closed_interm = 0;
    //total_connections = 0;

    if( connections > 0 ) {
        mergeFD( rdfds1, f);
    } // if
    
    connections = -1;

    return adjustment;
} // acceptBlock::getRead

void selectRead::getWrite(fd_set *fw, fd_set *fr ){
    waiting = true;
    do_plock dp(&l);
    waiting = false;

    if ( num_read_to_write > 0 ) { 
	mergeFD( wrfds1, fw);
	num_read_to_write = 0;
    } // if

    if ( num_write_to_read > 0 ) { 
	mergeFD( fr, rdfds );
	num_write_to_read = 0;
    } // if

    // not synching with acceptor so can't modify num_idle directly
    num_closed_interm += num_closed_intermW;
    num_closed_intermW = 0;

    if ( pthread_cond_signal( &wq ) != 0 ){
	printf("getRead signal error\n");
    }

} // selectRead::getWrite

void selectRead::mergeWrite( ){
    pthread_cond_wait( &wq, &l );
} // selectRead::mergeWrite

void writeQueue::readDrop(int num) {
    do_plock dp(&l);
    int sd;
    
    for (int i=0; i < num; i++) {	
	if ( epoll_write_results[i].events & EPOLLOUT ) {
	    sd = epoll_write_results[i].data.fd;
	    // epoll set for one shot, so don't change status
//	    interest_set_change( sd, ISET_NOT_WRITABLE );
	    queueEvents[frontEvents] = sd;
	    frontEvents = (frontEvents + 1)%maxconns;
	}
    } /* for */
    
    num_closed_intermW = num_closed_internal;
    num_closed_internal = 0;

    // use daisy-chain signalling to wake up the first writer task
    if ( num > 0 ) pthread_cond_signal( &writerWait );
    
} // writeQueue::readDrop


// tasks that go from readable to writable
// it may be reasonable to start active instaad
void writeQueue::readDrop_et(int sd) {
    do_plock dp(&l2);
    
    queueWritable[frontWritable] = sd;
    frontWritable = (frontWritable + 1)%maxconns;	    

    if( numTasks == writerWaitCount ) pthread_cond_signal( &writerWait );
} // writeQueue::readDrop_et

int writeQueue::writePickup(int old_sd) {
    do_plock dp(&l);

    num_closed_internal = num_closed_intermW;
    num_closed_intermW = 0;

    while ( frontEvents == backEvents ) pthread_cond_wait( &writerWait, &l);
    int ret = queueEvents[backEvents];
    backEvents = (backEvents + 1)%maxconns;

    // use daisy-chain signalling to wake up next writer if necessary
    if ( backEvents != frontEvents ) pthread_cond_signal( &writerWait );

    return ret;
} // writeQueue::writePickup

// use separate pickup for edge triggered as queues have different meanings
int writeQueue::writePickup_et(int old_sd, bool writeBlocked) {
    do_plock dp(&l);
    int ret = 0;

    // if putting something back, may require state change
    // it may be reasonable to always set it to blocked
    if( num_closed_intermW == 0 && old_sd > 0 ) {
	if( writeBlocked ) {
	    FD_SET(old_sd, &blockedTasks);
	} else {
	    queueWritable[frontWritable] = old_sd;
	    frontWritable = (frontWritable + 1)%maxconns;	    
	} // if
     } // if

    num_closed_internal += num_closed_intermW;
    num_closed_intermW = 0;

    while( ret == 0 ) {

	while ( frontEvents == backEvents && frontWritable == backWritable ) {
	    writerWaitCount += 1;
	    pthread_cond_wait( &writerWait, &l);
	    writerWaitCount -= 1;
	}	
	
	// check if something became writable
	struct info *ip = NULL;
	int sd = 0;
	for( ; frontEvents != backEvents && ret == 0; backEvents = (backEvents + 1)%maxconns ) {
	    sd = queueEvents[backEvents];
	    ip = info_ptr_no_assert(sd);
	    // TODO: there is a race condition here between closing the fd and it having edge events
	    if( ip != NULL ) ip->sd_write_state |= EPOLLOUT;
	    if ( FD_ISSET(sd, &blockedTasks) ) {
		FD_CLR(sd, &blockedTasks);
		ret = sd;
	    } // if
	} // for
	
	// check for new writables
	for( ; frontWritable != backWritable && ret == 0; backWritable = (backWritable + 1)%maxconns ) {
	    sd = queueWritable[backWritable];
	    ip = info_ptr(sd);
	    if (ip->sd_write_state & EPOLLOUT ) {
		ret = sd;
	    } else {
		FD_SET(sd, &blockedTasks);
	    } // if
	    
	}  // for

    } //while

    return ret;  
} // writeQueue::writePickup_et


// use separate pickup for edge triggered as queues have different meanings
void writeQueue::writeAdmin_et() {
    do_plock dp(&l);
    bool nothing = false;
    int affinity_queue = 0;
    bool writable = false;

    bool signal[numTasks];
    
    for( int i = 0; i < numTasks; i++ ) {
	signal[numTasks] = false;
    } // for

    while( true ) {
	// race condition between missing signals between loop iterations, a timeout maybe needed
	// might be better to move wait until after the return fd check

	if ( nothing ) {
	    pthread_cond_wait( &writerWait, &l);
	} else {
	    sched_yield();
	}// if

	nothing = true;

	// get return fds
	for( int i = 0; i < numTasks; i++ ) {
	    int old_sd = 0;
	    while ( true ){
		old_sd = FRQueue[i]->returnPickup(writable);
		if ( old_sd < 0 ) break;

		if ( !writable ) {
		    sd_write_state[old_sd] |= EPOLLOUT;
		    ReadSelQueue->readDropAgain_et(old_sd);
		} else {
		    // did event occur before sd is picked up?
		    if( sd_write_state[old_sd] & EPOLLOUT ) {
			sd_write_state[old_sd] &= ~(EPOLLOUT);
			affinity_queue = ReadSelQueue->Admin_info[old_sd].affinity_queue;
			FRQueue[affinity_queue]->forwardDrop(old_sd);
			signal[affinity_queue] = true;			
		    } else {
			FD_SET( old_sd, &blockedTasks);
		    } // if
		}
		nothing = false;
	    } //while
		    
	} // for

	// check if something became writable
	int sd = 0;
	for( ; frontEvents != backEvents; backEvents = (backEvents + 1)%maxconns ) {
	    nothing = false;
	    sd = queueEvents[backEvents];
	    if ( FD_ISSET(sd, &blockedTasks) ) {
		FD_CLR(sd, &blockedTasks);
		affinity_queue = ReadSelQueue->Admin_info[sd].affinity_queue;
		FRQueue[affinity_queue]->forwardDrop(sd);
		signal[affinity_queue] = true;
	    } else {
		sd_write_state[sd] |= EPOLLOUT;
	    } // if
	} // for

    } // while
} // writeQueue::writeAdmin_et


// SelectRead task drops off sds that have become readable
void readSelQueue::readDrop(int num) {
    extern struct epoll_event *epoll_read_results;
    do_plock dp(&l);
    int sd;
    int num_in = num;
    struct info * ip = NULL;
    
    // add readable fds to queue
    for (int i=0; i<num; i++) {
      sd = epoll_read_results[i].data.fd;
      uint32_t evt = epoll_read_results[i].events;

      if( epoll_is_hungup( evt ) ) {
	  /* epoll reported an error, so close the connection down */
	  ip = info_ptr(sd);
	  do_close(ip, true);
	  if (evt & EPOLLERR) num_epoll_epollerr++;
	  if (evt & EPOLLHUP) num_epoll_epollhup++;

      } else if ( epoll_is_readable( evt )) {
	    queueEvents[frontEvents] = sd;
	    frontEvents = (frontEvents + 1)%maxconns;
	}	
    } // for
    
    if ( num_in > 0 ) { 
	pthread_cond_broadcast( &readerWait );
	pthread_cond_wait( &readerWait1, &l );
    } // if

} // readSelQueue::readDrop

// called after accept so initial state is blocked
// it may be reasonable to start active instaad
void readSelQueue::readDropAgain_et(int sd) {
    do_plock dp(&l2);
    
    queueReadableAgain[frontReadableAgain] = sd;
    frontReadableAgain = (frontReadableAgain + 1)%maxconns;	    
    if (waiting) pthread_cond_broadcast( &readerWait );
} // readSelQueue::readDropAgain_et


// called after accept so initial state is blocked
// it may be reasonable to start active instaad
void readSelQueue::readDrop_et(int sd) {
    do_plock dp(&l2);

    queueReadable[frontReadable] = sd;
    frontReadable = (frontReadable + 1)%maxconns;
    if (waiting) pthread_cond_broadcast( &readerWait );
} // readSelQueue::readDrop_et

int readSelQueue::readPickup(int num) {
    do_plock dp(&l);

    while ( frontEvents == backEvents ) {
	pthread_cond_signal( &readerWait1 );
	pthread_cond_wait( &readerWait, &l);
    }
    int ret = queueEvents[backEvents];
    backEvents = (backEvents + 1)%maxconns;

    return ret;  
} // readSelQueue::readPickup

// use separate pickup for edge triggered as queues have different meanings
int readSelQueue::readPickup_et(int old_sd, bool readBlocked) {
    do_plock dp(&l);
    int ret = 0;

    // if putting something back, may require state change
    // it may be reasonable to always set it to blocked
    if( num_closed_intermW == 0 && old_sd > 0 ) {
	if( readBlocked ) {
	    FD_SET(old_sd, &blockedTasks);
	} else {
	    assert(false);
	    queueReadable[frontReadable] = old_sd;
	    frontReadable = (frontReadable + 1)%maxconns;	    
	} // if
    } // if

    num_closed_internal += num_closed_intermW;
    num_closed_intermW = 0;

    while( ret == 0 ) {

	while ( frontEvents == backEvents && frontReadable == backReadable ) {
	    pthread_cond_broadcast( &WriteQueue->writerWait );
	    pthread_cond_signal( &readerWait1 );
	    pthread_cond_wait( &readerWait, &l);
	}	
	
	// check if something becomes readable
	struct info *ip = NULL;
	int sd = 0;
	for( ; frontEvents != backEvents && ret == 0; backEvents = (backEvents + 1)%maxconns ) {
	    sd = queueEvents[backEvents];
	    ip = info_ptr_no_assert(sd);
	    // TODO: there is a race condition here between closing the fd and it having edge events
	    if(ip != NULL ) ip->sd_read_state |= EPOLLIN;
	    if ( FD_ISSET(sd, &blockedTasks) ) {
		FD_CLR(sd, &blockedTasks);
		ret = sd;
	    } // if
	} // for
	
	// check for new readables
	for( ; frontReadable != backReadable && ret == 0; backReadable = (backReadable + 1)%maxconns ) {
	    sd = queueReadable[backReadable];
	    ip = info_ptr(sd);
	    if (ip->sd_read_state & EPOLLIN ) {
		ret = sd;
	    } else {
		FD_SET(sd, &blockedTasks);
	    } // if
	    
	}  // for

	// check for new readables
	for( ; frontReadableAgain != backReadableAgain && ret == 0; backReadableAgain = (backReadableAgain + 1)%maxconns ) {
	    sd = queueReadableAgain[backReadableAgain];
	    ip = info_ptr(sd);
	    if (ip->sd_read_state & EPOLLIN ) {
		ret = sd;
	    } else {
		FD_SET(sd, &blockedTasks);
	    } // if
	    
	}  // for

    } //while
    return ret;  
} // readSelQueue::readPickup_et


// use separate pickup for edge triggered as queues have different meanings
void readSelQueue::readAdmin_et() {
    do_plock dp(&l);
    bool nothing = false;
    int affinity_queue = 0;
    struct admin_info *aip = NULL;
    bool writable = false;

    bool signal[numTasks];

    for( int i = 0; i < numTasks; i++ ) {
	signal[i] = false;
    } // for

    while( true ) {
	// race condition between missing signals between loop iterations, a timeout maybe needed
	// might be better to move wait until after the return fd check

	if ( nothing ) {
	    waiting = true;
	    pthread_cond_wait( &readerWait, &l);
	    waiting = false;
	} else {
	    sched_yield();
	}// if

	nothing = true;

	// get return fds
	for( int i = 0; i < numReaders; i++ ) {
	    int old_sd = 0;
	    while ( true ){
		old_sd = FRQueueReader[i]->returnPickup(writable);
		if ( old_sd < 0 ) break;

		if( writable ) {
		    WriteQueue->readDrop_et(old_sd);
		} else {
		    aip = &Admin_info[old_sd];
		    if (EPOLLIN ) {
			affinity_queue = aip->affinity_queue;
			FRQueueReader[affinity_queue]->forwardDrop(old_sd);
			signal[affinity_queue] = true;
		    } else {
			FD_SET( old_sd, &blockedTasks);
		    } //endif
		}
		nothing = false;
	    } //while
		    
	} // for

	// check if something becomes readable
	int sd = 0;
	for( ; frontEvents != backEvents; backEvents = (backEvents + 1)%maxconns ) {
	    nothing = false;
	    sd = queueEvents[backEvents];
	    aip = &Admin_info[sd];
	    if ( FD_ISSET(sd, &blockedTasks) ) {
		affinity_queue = aip->affinity_queue;
		FD_CLR(sd, &blockedTasks);
		FRQueueReader[affinity_queue]->forwardDrop(sd);
		signal[affinity_queue] = true;
	    } // if
	} // for

    } //while

} // readSelQueue::readAdmin_et

